﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Microsoft.Office.Interop.Excel;
using System.Diagnostics;
using System.Data;
using System.Data.OleDb;

namespace MW_HouseholdLedger
{
    public enum MWType
    {
        IMPORTMONEY,EXPORTMONEY,TOTALMONEY
    }
    public class MWExcel
    {
        private static MWExcel oExelInstance = null;
        private const String ExcelDirectoryPath = "DATA";
        private String ExcelFileName = "";
        private String m_sExcelFileFullPath = "";
        private const int m_nTotalSheetCount = 2;
        private String[] SheetName 
            = new String[m_nTotalSheetCount] 
            { "MWMoney", "MWCategory" };
        private String szConn = "";
        private DataSet m_cDataSet;
        OleDbConnection conn = null;

        public MWExcel() {
            StringBuilder szBuffer = new StringBuilder();
            MWGetINI.MWGetPrivateProfileString("DEFAULT", "ExcelFilename", "MWHouseholdLedger.xls", szBuffer, "MWExel.ini");

            ExcelFileName = szBuffer.ToString();

            DirectoryInfo DI = new DirectoryInfo(ExcelDirectoryPath);
            if (!DI.Exists) {
                DI.Create();
            }
            FileInfo FI = new FileInfo(ExcelDirectoryPath + "\\" + ExcelFileName);
            m_sExcelFileFullPath = FI.FullName;
            if(!FI.Exists){
                ExcelFileCreate(m_sExcelFileFullPath);
            }
            szConn = @"Provider=Microsoft.Jet.OLEDB.4.0; Data Source=" + m_sExcelFileFullPath + "; Extended Properties='Excel 8.0; HDR=YES';";
            conn = new OleDbConnection(szConn);
        }
        public System.Data.DataTable SelectData()
        {
            m_cDataSet = new DataSet();
            String sQuery = "";

            sQuery = String.Format("select Index,Type,Day,Category,Contents,Money from [{0}$]", SheetName[0]);
            OleDbDataAdapter adp = new OleDbDataAdapter();
            adp.SelectCommand = new OleDbCommand(sQuery, conn);
            adp.Fill(m_cDataSet);

            m_cDataSet.Tables[0].Columns[0].ColumnName = "No";
            m_cDataSet.Tables[0].Columns[1].ColumnName = "タイプ";
            m_cDataSet.Tables[0].Columns[2].ColumnName = "日時";
            m_cDataSet.Tables[0].Columns[3].ColumnName = "カテゴリ";
            m_cDataSet.Tables[0].Columns[4].ColumnName = "内容";
            m_cDataSet.Tables[0].Columns[5].ColumnName = "金額";

            for (int i = 0; i < m_cDataSet.Tables[0].Rows.Count; i++)
            {
                if (m_cDataSet.Tables[0].Rows[i][1].ToString().Trim().Equals("0"))
                {
                    m_cDataSet.Tables[0].Rows[i][1] = "入金";
                }
                else if (m_cDataSet.Tables[0].Rows[i][1].ToString().Trim().Equals("1"))
                {
                    m_cDataSet.Tables[0].Rows[i][1] = "出勤";
                }

                m_cDataSet.Tables[0].Rows[i][5] = cMoneyFormatString(new StringBuilder(m_cDataSet.Tables[0].Rows[i][5].ToString()));
            }
            return m_cDataSet.Tables[0];
        }
        public void DataDispose() 
        {
            m_cDataSet.Dispose();
            m_cDataSet = null;
        }
        public StringBuilder cMoneyFormatString(StringBuilder sStringData)
        {
            for (int i = sStringData.Length, j = 0; i > 0; i--, j++)
            {
                if (j != 0 && j % 3 == 0)
                {
                    sStringData.Insert(i, ",");
                }
            }
            return sStringData;
        }
        private void ExcelFileCreate(String pFileName)
        {
            MWLogWriter.Instance().LogWrite("Excel 파일 생성");
            int excelSheetCount = 0;

            Application excelApp = null;
            Workbook excelBook = null;
            Worksheet excelWorksheet = null;

            try
            {
                object missingType = Type.Missing;
                excelApp = new Application();
                excelApp.Visible = false;
                excelApp.DisplayAlerts = false;

                excelBook = excelApp.Workbooks.Add(missingType);
                excelSheetCount = excelBook.Sheets.Count;

                for (int sheetIndex = 1; sheetIndex <= m_nTotalSheetCount; sheetIndex++)
                {
                    if (excelSheetCount < sheetIndex)
                    {
                        excelWorksheet = (Worksheet)excelBook.Worksheets.Add(missingType, missingType, missingType, missingType);
                    }
                    else
                    {
                        excelWorksheet = (Worksheet)excelBook.Sheets.get_Item(sheetIndex);
                    }
                    if (sheetIndex == 1 ) {
                        excelWorksheet.Cells[1, 1] = "Index";
                        excelWorksheet.Cells[1, 2] = "Type";
                        excelWorksheet.Cells[1, 3] = "Category";
                        excelWorksheet.Cells[1, 4] = "Day";
                        excelWorksheet.Cells[1, 5] = "Contents";
                        excelWorksheet.Cells[1, 6] = "Money";
                    }
                    else if (sheetIndex == 2) {
                        excelWorksheet.Cells[1, 1] = "No";
                        excelWorksheet.Cells[1, 2] = "Type";
                        excelWorksheet.Cells[1, 3] = "Category";
                    }
                    excelWorksheet.Name = SheetName[sheetIndex - 1];
                }

                excelBook.SaveAs(pFileName, XlFileFormat.xlWorkbookNormal, missingType, missingType, missingType, missingType, XlSaveAsAccessMode.xlExclusive, missingType, missingType, missingType, missingType, missingType);
                System.Runtime.InteropServices.Marshal.ReleaseComObject(excelApp);
                MWLogWriter.Instance().LogWrite("생성이 완료되었습니다.");
            }
            catch (Exception e)
            {
                MWLogWriter.Instance().LogWrite("Excel 파일 생성 중 에러가 발생하였습니다.");
                MWLogWriter.Instance().LogWrite(e.ToString());
            }
            finally
            {
                excelBook.Close();
                excelApp = null;
            }
        }
        
        public bool InsertData(MWType nType, String sCategory, String sDate, String sContents, String sMoney)
        {
            
            int psType = 0;
            String strQy;
            int count = TableIndex(new StringBuilder(SheetName[0]));
            if (nType == MWType.IMPORTMONEY)
            {
                psType = 0;
            }
            else if (nType == MWType.EXPORTMONEY)
            {
                psType = 1;
            }
            else
            {
                return false;
            }
            OleDbCommand cmd = new OleDbCommand();
            cmd.Connection = conn;
            strQy = string.Format("insert into [{0}$] values('{1}','{2}','{3}','{4}','{5}','{6}')", SheetName[0], count, psType, sCategory, sDate, sContents, sMoney);
            cmd.CommandText = strQy;
            conn.Open();
            cmd.ExecuteNonQuery();
            conn.Close();

            return true;
        }
        private int TableIndex(StringBuilder sTableName)
        {
            int nCount = 0;
            OleDbCommand cmd = new OleDbCommand();
            cmd.Connection = conn;
            cmd.CommandText = String.Format("select count(*) from [{0}$]", sTableName);
            conn.Open();
            OleDbDataReader dr = cmd.ExecuteReader();
            if (dr.Read())
            {
                nCount = Convert.ToInt32(dr.GetValue(0)) + 1;
            }
            dr.Dispose();
            conn.Close();
            return nCount;
        }
        public static MWExcel Instance()
        {
            if (oExelInstance == null) {
                oExelInstance = new MWExcel();
            }
            return oExelInstance;
        }
        public int[] TotalData() 
        {
            int[] nTotalData = new int[2];
            OleDbCommand cmd = new OleDbCommand();
            cmd.Connection = conn;

            for (int i = 0; i < 2; i++) {

                cmd.CommandText = String.Format("select sum(Money) from [{0}$] where type='{1}'", SheetName[0],i);
                conn.Open();
                OleDbDataReader dr = cmd.ExecuteReader();
                if (dr.Read())
                {
                    nTotalData[i] = Convert.ToInt32(dr.GetValue(0));
                }
                dr.Dispose();
                conn.Close();
            }

            return nTotalData;
        }
    }
}
