﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Globalization;

namespace MW_HouseholdLedger
{
    public partial class MW_DataInputPanel : UserControl
    {
        private int SelectIndex = -1;

        public MW_DataInputPanel()
        {
            InitializeComponent();
        }
        CultureInfo cLanguage;
        ImeMode cImeMode;
        private void MW_DataInputPanel_Load(object sender, EventArgs e)
        {
            MWWINAPI.InPutPanelHandle = Handle;
            Initialize();
            cLanguage = new CultureInfo("ja");
            InputLanguage.FromCulture(cLanguage);
            cImeMode = ImeMode.Alpha;
        }
        private void Initialize() {
            SelectIndex = -1;

            this.cInsertButton.Visible = true;
            this.cModifyButton.Visible = false;
            this.cDeleteButton.Visible = false;
            this.cCancelButton.Visible = false;

            DateTime cDateTimeNow = DateTime.Now;
            TypeInitialize();
            DateInitialize(cDateTimeNow.Year, cDateTimeNow.Month, cDateTimeNow.Day);
            this.cContentsTextbox.Text = "";
            this.cMoneyTextbox.Text = "";
        }
        private void TypeInitialize()
        {
            String[] sType = new string[] {"入金","出金"};
            object[] m_TypeData = new object[2];
            this.cTypeCombo.Items.Clear();
            for (int i = 0; i < 2; i++) 
            {
                m_TypeData[i] = sType[i];
            }
            this.cTypeCombo.Items.AddRange(m_TypeData);
            this.cTypeCombo.Text = this.cTypeCombo.Items[0].ToString();

            sType = null;
        }
        private void DateInitialize(int nYear, int nMonth, int nDay) 
        {
            this.cDateYearCombo.Items.Clear();

            if (this.cDateYearCombo.Items.Count == 0)
            {
                object[] m_YearData = new object[5];
                for (int i = 0; i < 5; i++)
                {
                    m_YearData[i] = (2013 + i).ToString("00");
                }
                this.cDateYearCombo.Items.AddRange(m_YearData);
            }

            if (this.cDateMonthCombo.Items.Count == 0)
            {
                this.cDateMonthCombo.Items.Clear();
                object[] m_MonthData = new object[12];
                for (int i = 0; i < 12; i++)
                {
                    m_MonthData[i] = (i + 1).ToString("00");
                }
                this.cDateMonthCombo.Items.AddRange(m_MonthData);
            }
            DayCalculater(nYear, nMonth);

            this.cDateYearCombo.Text = nYear.ToString("0000");
            this.cDateMonthCombo.Text = nMonth.ToString("00");
            this.cDateDayCombo.Text = nDay.ToString("00");

        }

        private void DayCalculater(int nYear,int nMonth) {
            int nDay = 1;
            if (this.cDateDayCombo.Text.Trim() != String.Empty)
            {
                nDay = Convert.ToInt32(this.cDateDayCombo.Text);
            }
            this.cDateDayCombo.Items.Clear();
            int nDayMax = 0;
            switch (nMonth)
            {
                case 1:
                    nDayMax = 31;
                    break;

                case 2:
                    nDayMax =
                        ((nYear % 4 == 0 && nYear % 100 != 0) || nYear % 400 == 0)
                            ? 29 : 28;
                    break;

                case 3:
                    nDayMax = 31;
                    break;

                case 4:
                    nDayMax = 30;
                    break;

                case 5:
                    nDayMax = 31;
                    break;

                case 6:
                    nDayMax = 30;
                    break;

                case 7:
                    nDayMax = 31;
                    break;

                case 8:
                    nDayMax = 31;
                    break;

                case 9:
                    nDayMax = 30;
                    break;

                case 10:
                    nDayMax = 31;
                    break;

                case 11:
                    nDayMax = 30;
                    break;

                case 12:
                    nDayMax = 31;
                    break;
            }

            object[] m_DayData = new object[nDayMax];
            for (int i = 0; i < nDayMax; i++)
            {
                m_DayData[i] = (i + 1).ToString("00");
            }
            this.cDateDayCombo.Items.AddRange(m_DayData);

            if (nDay > nDayMax)
            {
                this.cDateDayCombo.Text = this.cDateDayCombo.Items[0].ToString();
            }
            else 
            {
                this.cDateDayCombo.Text = nDay.ToString();
            }
        }

        private void cDateMonthCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.cDateYearCombo.Text.Trim() != String.Empty && this.cDateMonthCombo.Text.Trim() != String.Empty) 
            {
                DayCalculater(Convert.ToInt32(this.cDateYearCombo.Text), Convert.ToInt32(this.cDateMonthCombo.Text));
            }
        }

        private void cDateYearCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (this.cDateYearCombo.Text.Trim() != String.Empty && this.cDateMonthCombo.Text.Trim() != String.Empty)
            {
                DayCalculater(Convert.ToInt32(this.cDateYearCombo.Text), Convert.ToInt32(this.cDateMonthCombo.Text));
            }
        }

        private void cMoneyTextbox_KeyDown(object sender, KeyEventArgs e)
        {
            StringBuilder sBuffer;
            if ((e.KeyCode >= Keys.D0 && e.KeyCode <= Keys.D9) || (e.KeyCode >= Keys.NumPad0 && e.KeyCode <= Keys.NumPad9))
            {
                int nCusorPosition = cMoneyTextbox.SelectionStart + 1;
                sBuffer = new StringBuilder(this.cMoneyTextbox.Text);
                if (e.KeyCode >= Keys.D0 && e.KeyCode <= Keys.D9)
                {
                    sBuffer.Insert(nCusorPosition - 1, e.KeyValue - 48);
                }
                else
                {
                    sBuffer.Insert(nCusorPosition - 1, e.KeyValue - 96);
                }
                
                nCusorPosition += cMoneyFormatString(sBuffer);
                this.cMoneyTextbox.Text = sBuffer.ToString();
                this.cMoneyTextbox.Select(nCusorPosition, 0);
            }
            else if (e.KeyCode == Keys.Back)
            {
                int nCusorPosition = cMoneyTextbox.SelectionStart;
                if (nCusorPosition > 0)
                {
                    if (this.cMoneyTextbox.SelectionLength == 0)
                    {
                        sBuffer = new StringBuilder(this.cMoneyTextbox.Text);
                        sBuffer.Remove(nCusorPosition - 1, 1);
                        nCusorPosition += cMoneyFormatString(sBuffer);
                        this.cMoneyTextbox.Text = sBuffer.ToString();
                        if (nCusorPosition - 1 < 0) 
                        {
                            nCusorPosition = 1;
                        }
                        if (nCusorPosition > 1 &&sBuffer[nCusorPosition - 2].Equals(','))
                        {
                            nCusorPosition--;
                        }
                        this.cMoneyTextbox.Select(nCusorPosition - 1, 0);
                    }
                    else
                    {
                        int nPosition = cMoneyTextbox.SelectionStart;
                        sBuffer = new StringBuilder(this.cMoneyTextbox.Text);
                        sBuffer.Remove(nPosition, cMoneyTextbox.SelectionLength);
                        nCusorPosition += cMoneyFormatString(sBuffer);
                        this.cMoneyTextbox.Text = sBuffer.ToString();
                        this.cMoneyTextbox.Select(nPosition, 0);
                        
                    }
                }
            }
            else if (e.KeyCode == Keys.Delete) {
                int nCusorPosition = cMoneyTextbox.SelectionStart;
                if (this.cMoneyTextbox.SelectionLength == 0)
                {
                    if (nCusorPosition < cMoneyTextbox.Text.Length)
                    {
                        sBuffer = new StringBuilder(this.cMoneyTextbox.Text);
                        sBuffer.Remove(nCusorPosition, 1);
                        nCusorPosition += cMoneyFormatString(sBuffer);
                        this.cMoneyTextbox.Text = sBuffer.ToString();
                        if (nCusorPosition < 0)
                        {
                            nCusorPosition = 0;
                        }
                        this.cMoneyTextbox.Select(nCusorPosition, 0);
                    }
                }
                else
                {
                    int nPosition = cMoneyTextbox.SelectionStart;
                    sBuffer = new StringBuilder(this.cMoneyTextbox.Text);
                    sBuffer.Remove(nPosition, cMoneyTextbox.SelectionLength);
                    nCusorPosition += cMoneyFormatString(sBuffer);
                    this.cMoneyTextbox.Text = sBuffer.ToString();
                    this.cMoneyTextbox.Select(nPosition, 0);
                }
            }
            if (e.KeyCode == Keys.Left || e.KeyCode == Keys.Right || e.KeyCode == Keys.End || e.KeyCode == Keys.Home)
            {
                e.Handled = false;
            }
            else
            {
                e.Handled = true;
            }
        }
        private void cMoneyTextbox_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = true;
        }
        private int cMoneyFormatString(StringBuilder sStringData)
        {
            int nPreLenth = sStringData.Length;
            sStringData = sStringData.Replace(",", "");
            for (int i = sStringData.Length, j = 0; i > 0; i--, j++)
            {
                if (j != 0 && j % 3 == 0)
                {
                    sStringData.Insert(i, ",");
                }
            }
            return sStringData.Length - nPreLenth;
        }
        private void cInsertButton_Click(object sender, System.EventArgs e) //등록
        {
            if (cContentsTextbox.Text.Trim() == String.Empty || cMoneyTextbox.Text.Trim() == String.Empty)
            {
                return;
            }

            MWType nType;
            if (cTypeCombo.SelectedIndex == 0)
            {
                nType = MWType.IMPORTMONEY;
            }
            else if (cTypeCombo.SelectedIndex == 1)
            {
                nType = MWType.EXPORTMONEY;
            }
            else
            {
                return;
            }

            String sDate = String.Format("{0:0000}{1:00}{2:00}", Convert.ToInt32(cDateYearCombo.Text), Convert.ToInt32(cDateMonthCombo.Text), Convert.ToInt32(cDateDayCombo.Text));
            int idx = MWSqlDB.Instance().DataInsert(nType, "1", sDate, cContentsTextbox.Text.Trim(), cMoneyTextbox.Text.Replace(",", "").Trim());

            String Message = String.Format("{0}|{1}|{2}|{3}|{4}|{5}|{6}", 
                                            idx,
                                            cTypeCombo.Text, 
                                            cCategoryCombo.Text, 
                                            sDate, 
                                            cContentsTextbox.Text.Trim(), 
                                            cMoneyTextbox.Text.Replace(",", "").Trim(),
                                            cTypeCombo.SelectedIndex
                                            );


            MWWINAPI.MWSendMessage(MWWINAPI.DataViewHandle, MWMessage.ItemInsert, new StringBuilder(Message));

            sDate = String.Empty;
            Message = String.Empty;

            Initialize();
        }
        protected override void WndProc(ref Message m)
        {
            switch ((MWMessage)m.Msg)
            {
                case MWMessage.ItemSelect:
                    Encoding euc = Encoding.GetEncoding("Shift_JIS");
                    int nMessageLength = MWWINAPI.MessageLength(m.WParam);
                    byte[] nMessageBuf = new byte[nMessageLength];
                    Marshal.Copy(m.LParam, nMessageBuf, 0, nMessageLength);
                    DataGrideSelected(euc.GetString(nMessageBuf));
                    nMessageBuf = null;
                    break;
            }
            base.WndProc(ref m);
        }
        String sPreTypeState = "";
        String sPreMoney = "";
        private void DataGrideSelected(String MessageData)
        {
            String[] sData = MessageData.Split('|');
            String[] sDate = sData[3].Split('-');
            SelectIndex = Convert.ToInt32(sData[0]);
            cTypeCombo.Text = sData[1].Trim();
            cCategoryCombo.Text = "";    //카테고리 미완료
            cDateYearCombo.Text = sDate[0];
            cDateMonthCombo.Text = sDate[1];
            cDateDayCombo.Text = sDate[2];

            cContentsTextbox.Text = sData[4];
            cMoneyTextbox.Text = sData[5];

            sPreTypeState = cTypeCombo.SelectedIndex.ToString();
            sPreMoney = sData[5].Replace(",", "");

            sData = null;
            sDate = null;

            this.cInsertButton.Visible = false;
            this.cModifyButton.Visible = true;
            this.cDeleteButton.Visible = true;
            this.cCancelButton.Visible = true;
        }

        private void cCancelButton_Click(object sender, EventArgs e)
        {
            Initialize();
        }

        private void cDeleteButton_Click(object sender, EventArgs e)
        {
            
            if (SelectIndex != -1)
            {
                MWSqlDB.Instance().DataDelete(SelectIndex.ToString());
                String sMessage = String.Format("{0}|{1}|{2}", SelectIndex, sPreTypeState, sPreMoney);
                MWWINAPI.MWSendMessage(MWWINAPI.DataViewHandle, MWMessage.ItemDelete, new StringBuilder(sMessage));
            }
            Initialize();
        }

        private void cModifyButton_Click(object sender, EventArgs e)
        {
            if (SelectIndex != -1)
            {
                MWType nType;

                String sDate = String.Format("{0:0000}{1:00}{2:00}", Convert.ToInt32(cDateYearCombo.Text), Convert.ToInt32(cDateMonthCombo.Text), Convert.ToInt32(cDateDayCombo.Text));
                if (cTypeCombo.SelectedIndex == 0)
                {
                    nType = MWType.IMPORTMONEY;
                }
                else if (cTypeCombo.SelectedIndex == 1)
                {
                    nType = MWType.EXPORTMONEY;
                }
                else
                {
                    return;
                }

                MWSqlDB.Instance().DataModify(SelectIndex.ToString(), nType, "1", sDate, cContentsTextbox.Text.Trim(), cMoneyTextbox.Text.Replace(",", "").Trim());
                
                String sMessage = String.Format("{0}|{1}|{2}|{3}|{4}|{5}|{6}|{7}|{8}",
                                                SelectIndex,
                                                cTypeCombo.Text,
                                                cCategoryCombo.Text,
                                                sDate,
                                                cContentsTextbox.Text.Trim(),
                                                cMoneyTextbox.Text.Replace(",", "").Trim(),
                                                cTypeCombo.SelectedIndex,
                                                sPreTypeState,
                                                sPreMoney
                                                );
                MWWINAPI.MWSendMessage(MWWINAPI.DataViewHandle, MWMessage.ItemModify, new StringBuilder(sMessage));
            }
            Initialize();
        }

        private void cContentsTextbox_Enter(object sender, EventArgs e)
        {
            InputLanguage.FromCulture(cLanguage);
            cContentsTextbox.ImeMode = ImeMode;
        }

        private void cContentsTextbox_Leave(object sender, EventArgs e)
        {
            cLanguage = CultureInfo.CurrentCulture;
            ImeMode = cContentsTextbox.ImeMode;
        }

        private void cMoneyTextbox_Enter(object sender, EventArgs e)
        {
            cMoneyTextbox.ImeMode = System.Windows.Forms.ImeMode.Alpha;
            cMoneyTextbox.ImeMode = System.Windows.Forms.ImeMode.Disable;
        }

        
    }
}
