﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace MW_HouseholdLedger
{
    /// <summary>
    /// 일반의 검색 조건 및 그리드 뷰
    /// </summary>
    public partial class MW_DataView : UserControl
    {
        /// <summary>
        /// 생성자 - 디자인 및 이벤트 세팅
        /// </summary>
        /// 

        public MW_DataView()
        {
            MWLogWriter.Instance().LogWrite("DataView가 기동합니다.");
            InitializeComponent();
        }

        /// <summary>
        /// 폼 로드 이벤트
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// 

        private void MW_DataView_Load(object sender, EventArgs e)
        {
            MWWINAPI.DataViewHandle = Handle;
            TypeComboInitailize();      //타입
            CategoryInitailize();       //카테고리
            DateInitialize();           //날짜

            TotalInitialize();
            MWLogWriter.Instance().LogWrite("초기화가 완료됩니다.");
        }

        /// <summary>
        /// 입, 출금 콤보박스 초기화
        /// </summary>
        /// 

        private void TypeComboInitailize()
        {
            String[] sType = new string[] { "全体", "入金", "出金" };
            object[] m_TypeData = new object[3];
            this.cTypeCombo.Items.Clear();
            for (int i = 0; i < 3; i++)
            {
                m_TypeData[i] = sType[i];
            }
            this.cTypeCombo.Items.AddRange(m_TypeData);
            this.cTypeCombo.Text = this.cTypeCombo.Items[0].ToString();

            sType = null;
            m_TypeData = null;
        }


        /// <summary>
        /// 셀 클릭 이벤트
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e">
        /// e.RowIndex = 셀 위치 값
        /// -1 값일시에는 정렬 클릭임
        /// </param>
        /// 

        private void cDataGrid_CellMouseClick(object sender, DataGridViewCellMouseEventArgs e)
        {
            if (e.RowIndex != -1)
            {
                int nRowIndex = e.RowIndex;
                String Message = String.Format("{0}|{1}|{2}|{3}|{4}|{5}",
                                                cDataGrid.Rows[nRowIndex].Cells[0].EditedFormattedValue,
                                                cDataGrid.Rows[nRowIndex].Cells[1].EditedFormattedValue,
                                                cDataGrid.Rows[nRowIndex].Cells[3].EditedFormattedValue,
                                                cDataGrid.Rows[nRowIndex].Cells[2].EditedFormattedValue,
                                                cDataGrid.Rows[nRowIndex].Cells[4].EditedFormattedValue,
                                                cDataGrid.Rows[nRowIndex].Cells[5].EditedFormattedValue
                                             );
                MWWINAPI.MWSendMessage(MWWINAPI.InPutPanelHandle, MWMessage.ItemSelect, new StringBuilder(Message));
            }
        }

        /// <summary>
        /// 데이터가 추가 되면 선택되어진다.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// 
        private void cDataGrid_RowsAdded(object sender, DataGridViewRowsAddedEventArgs e)
        {
            cDataGrid.ClearSelection();
            cDataGrid.Rows[cDataGrid.Rows.Count - 1].Selected = true;
            cDataGrid.FirstDisplayedScrollingRowIndex = cDataGrid.Rows.Count - 1;
        }

        /// <summary>
        /// 시작되면 처음 최초 검색을 실시한다.
        /// </summary>
        /// 

        private void TotalInitialize()
        {
            SearchData();
        }

        /// <summary>
        /// 데이터 베이스로 부터 검색하여 값을 세팅한다.
        /// </summary>
        /// 

        public void SearchData() 
        {
            Encoding enc = Encoding.GetEncoding("Shift_JIS");
            StringBuilder where = new StringBuilder(" and ");
            switch (cTypeCombo.SelectedIndex) 
            {
                case 0: where.Append(" 1 = 1"); break;
                case 1: where.Append(" a.MoneyType = 1 "); break;
                case 2: where.Append(" a.MoneyType = 2 "); break;
            }

            String sDate = String.Format(   "{0:0000}{1:00}{2:00}"                      , 
                                                Convert.ToInt32(cSDateYearCombo.Text)   ,
                                                Convert.ToInt32(cSDateMonthCombo.Text)  ,  
                                                Convert.ToInt32(cSDateDayCombo.Text)
                                        );


            String eDate = String.Format(   "{0:0000}{1:00}{2:00}"                      , 
                                                Convert.ToInt32(cEDateYearCombo.Text)   , 
                                                Convert.ToInt32(cEDateMonthCombo.Text)  , 
                                                Convert.ToInt32(cEDateDayCombo.Text)
                                        );


            Decimal[] nTotalMoney = MWSqlDB.Instance().DataSelect(sDate, eDate, where, cDataGrid);

            cInputTotalText.Text = nTotalMoney[0].ToString("###,###");
            cOutputTotalText.Text = nTotalMoney[1].ToString("###,###");
            cTotalText.Text = (nTotalMoney[0] - nTotalMoney[1]).ToString("###,###");

            String[] HearderText = new String[] 
                                    { "No", "タイプ", "日時", "カテゴリ", "内容", "金額" };

            int [] CellSize = new int[] 
                                    { 30 ,50 ,70,60,155,100 };

            cDataGrid.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;

            for (int i = 0; i < cDataGrid.Columns.Count; i++)
            {
                cDataGrid.Columns[i].HeaderText = HearderText[i];
                //마지막 컬럼 - 가운데 정렬 및 돈 표시
                if (i == cDataGrid.Columns.Count - 1)
                {
                    cDataGrid.Columns[i].CellTemplate.Style.Alignment = DataGridViewContentAlignment.MiddleRight;
                    cDataGrid.Columns[i].DefaultCellStyle.Format = "###,###";
                    
                }
                //오른쪽 정렬
                else
                {
                    //두번째는 날짜 표시
                    if (i == 2) 
                    {
                        cDataGrid.Columns[i].DefaultCellStyle.Format = "####-##-##";
                    }
                    cDataGrid.Columns[i].CellTemplate.Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
                }
                cDataGrid.Columns[i].Width = CellSize[i];
            }

            MWLogWriter.Instance().LogWrite("cDataGrid View로 검색이 완료 되었습니다.");

            //메모리 해제
            nTotalMoney = null;
            HearderText = null;
            CellSize = null;
            enc = null;
            where = null;
           
        }

        /// <summary>
        /// 검색 날짜 초기화
        /// </summary>
        /// 
        private void DateInitialize()
        {
            DateTime cEDateTimeNow = DateTime.Now;
            DateTime cSDateTimeNow = cEDateTimeNow.AddMonths(-3);
            //연도 세팅
            this.cSDateYearCombo.Items.Clear();
            this.cEDateYearCombo.Items.Clear();
            
            object[] m_YearData = new object[5];
            for (int i = 0; i < 5; i++)
            {
                m_YearData[i] = 2012 + i;
            }
            this.cSDateYearCombo.Items.AddRange(m_YearData);
            this.cEDateYearCombo.Items.AddRange(m_YearData);
            m_YearData = null;
            //월 세팅
            this.cSDateMonthCombo.Items.Clear();
            this.cEDateMonthCombo.Items.Clear();
           
            object[] m_MonthData = new object[12];
            for (int i = 0; i < 12; i++)
            {
                m_MonthData[i] = i + 1;
            }
            this.cSDateMonthCombo.Items.AddRange(m_MonthData);
            this.cEDateMonthCombo.Items.AddRange(m_MonthData);
            m_MonthData = null;
            //일 세팅
            this.cSDateDayCombo.Items.Clear();
            this.cEDateDayCombo.Items.Clear();

            for(int i=1;i<=DayCalculater(cSDateTimeNow.Year, cSDateTimeNow.Month);i++)
            {
                this.cSDateDayCombo.Items.Add(i);
            }

            for(int i=1;i<=DayCalculater(cEDateTimeNow.Year, cEDateTimeNow.Month);i++)
            {
                this.cEDateDayCombo.Items.Add(i);
            }
            this.cSDateDayCombo.Text = cSDateTimeNow.Day.ToString();
            this.cEDateDayCombo.Text = cEDateTimeNow.Day.ToString();

            this.cSDateMonthCombo.Text = cSDateTimeNow.Month.ToString();
            this.cEDateMonthCombo.Text = cEDateTimeNow.Month.ToString();

            this.cSDateYearCombo.Text = cSDateTimeNow.Year.ToString();
            this.cEDateYearCombo.Text = cEDateTimeNow.Year.ToString();
        }

        /// <summary>
        /// 날짜 일 계산함수 (연,월 로 28,30,31 과 윤달을 계산한다.)
        /// </summary>
        /// <param name="nYear">연</param>
        /// <param name="nMonth">월</param>
        /// <returns>해당 월의 총 날짜 수</returns>
        /// 

        private int DayCalculater(int nYear, int nMonth)
        {
            int nDayMax = 0;
            switch (nMonth)
            {
                case 1:
                    nDayMax = 31;
                    break;

                case 2:
                    nDayMax =
                        ((nYear % 4 == 0 && nYear % 100 != 0) || nYear % 400 == 0)
                            ? 29 : 28;
                    break;

                case 3:
                    nDayMax = 31;
                    break;

                case 4:
                    nDayMax = 30;
                    break;

                case 5:
                    nDayMax = 31;
                    break;

                case 6:
                    nDayMax = 30;
                    break;

                case 7:
                    nDayMax = 31;
                    break;

                case 8:
                    nDayMax = 31;
                    break;

                case 9:
                    nDayMax = 30;
                    break;

                case 10:
                    nDayMax = 31;
                    break;

                case 11:
                    nDayMax = 30;
                    break;

                case 12:
                    nDayMax = 31;
                    break;
            }
            return nDayMax;
        }

        /// <summary>
        /// 카테고리 콤보박스 초기화 - 카테고리 부분은 설계중
        /// </summary>
        /// 

        private void CategoryInitailize()
        {
            int sTypebuf = this.cTypeCombo.SelectedIndex;
            //아직 안만듦
        }
        
        /// <summary>
        /// 연,월이 바뀔 때마다 일 콤보박스 날짜 재계산 함수
        /// </summary>
        /// <param name="nYear">연</param>
        /// <param name="nMonth">월</param>
        /// <param name="cDay">일 콤보 박스</param>
        /// 

        private void DayChange(int nYear,int nMonth,ComboBox cDay) 
        {
            String buf = cDay.Text;
            int maxDay = DayCalculater(nYear, nMonth);
            cDay.Items.Clear();
            for (int i = 1; i <= maxDay; i++)
            {
                cDay.Items.Add(i);
            }
            if (Convert.ToInt32(buf) > maxDay)
            {
                cDay.Text = "1";
            }
            else
            {
                cDay.Text = buf;
            }
            buf = null;
        }

        /// <summary>
        /// 메시지 루프 - 입력단으로 부터 입력, 수정, 삭제의 전문을 수신한다.
        /// </summary>
        /// <param name="m"></param>
        /// 

        protected override void WndProc(ref Message m)
        {
            switch ((MWMessage)m.Msg) 
            {
                case MWMessage.ItemInsert:
                    {
                        Encoding euc = Encoding.GetEncoding("Shift_JIS");
                        int nMessageLength = MWWINAPI.MessageLength(m.WParam);
                        byte[] nMessageBuf = new byte[nMessageLength];
                        Marshal.Copy(m.LParam, nMessageBuf, 0, nMessageLength);
                        DataGridInsert(euc.GetString(nMessageBuf));
                        nMessageBuf = null;
                        euc = null;
                       
                    }
                    break;
                case MWMessage.ItemDelete:
                    {
                        Encoding euc = Encoding.GetEncoding("Shift_JIS");
                        int nMessageLength = MWWINAPI.MessageLength(m.WParam);
                        byte[] nMessageBuf = new byte[nMessageLength];
                        Marshal.Copy(m.LParam, nMessageBuf, 0, nMessageLength);
                        DataGridDelete(euc.GetString(nMessageBuf).Trim());
                        nMessageBuf = null;
                        euc = null;
                    }
                    break;
                case MWMessage.ItemModify:
                    {
                        Encoding euc = Encoding.GetEncoding("Shift_JIS");
                        int nMessageLength = MWWINAPI.MessageLength(m.WParam);
                        byte[] nMessageBuf = new byte[nMessageLength];
                        Marshal.Copy(m.LParam, nMessageBuf, 0, nMessageLength);
                        DataGridModify(euc.GetString(nMessageBuf).Trim());
                        nMessageBuf = null;
                        euc = null;
                    }
                    break;
            }
            base.WndProc(ref m);
        }

        /// <summary>
        /// 데이터 수정이 발생 할 경우 데이터 그리드 값을 수정한다.
        /// </summary>
        /// <param name="MessageData">메시지 전문</param>
        /// 

        private void DataGridModify(String MessageData)
        {
            MWLogWriter.Instance().LogWrite("전문 수신 Msg : " + MessageData);
            Encoding enc = Encoding.GetEncoding("Shift_JIS");
            String[] Data = MessageData.Split('|');

            DataGridViewRow cRow = cDataGrid.SelectedRows[0];
            cRow.Cells[1].Value = Data[1];
            cRow.Cells[2].Value = Convert.ToInt32(Data[3]);
            cRow.Cells[3].Value = Data[2];
            cRow.Cells[4].Value = Data[4];
            cRow.Cells[5].Value = Data[5];

            Decimal MoneyBuf = Convert.ToDecimal(Data[8]);
            if (Data[7].Equals("0"))
            {
                TotalCalculater(3, cInputTotalText, cTotalText, MoneyBuf);
            }
            else if (Data[7].Equals("1"))
            {
                TotalCalculater(4, cOutputTotalText, cTotalText, MoneyBuf);
            }

            MoneyBuf = Convert.ToDecimal(Data[5]);
            if (Data[6] == "0")
            {
                TotalCalculater(1, cInputTotalText, cTotalText, MoneyBuf);
            }
            else if (Data[6] == "1")
            {
                TotalCalculater(2, cOutputTotalText, cTotalText, MoneyBuf);
            }

            MWLogWriter.Instance().LogWrite("cDataGrid View 데이터가 수정되었습니다. Data Index : " + Data[0]);
            //메모리 해제
            enc = null;
            Data = null;
            cRow = null;
            
        }

        /// <summary>
        /// 합계 텍스트 박스 계산 함수
        /// </summary>
        /// <param name="type">
        /// 1 - 입금 내용이 추가
        /// 2 - 출금 내용이 추가
        /// 3 - 입금 내용이 삭제
        /// 4 - 출금 내용이 삭제
        /// </param>
        /// <param name="SubTextBox">
        /// 바뀔 텍스트 박스
        /// </param>
        /// <param name="TotalTextBox">
        /// 전체 텍스트 박스
        /// </param>
        /// <param name="CalCulData">
        /// 변경될 데이터 값
        /// </param>
        /// 

        private void TotalCalculater(int type,TextBox SubTextBox,TextBox TotalTextBox,Decimal CalCulData)
        {
            Decimal[] nMoneyBuf = new Decimal[2];
            if (SubTextBox.Text.Trim() == String.Empty)
            {
                SubTextBox.Text = "0";
            }
            if (TotalTextBox.Text.Trim() == String.Empty)
            {
                TotalTextBox.Text = "0";
            }
            switch (type)
            {
                case 1:     //입금 증가
                    nMoneyBuf[0] = Convert.ToDecimal(SubTextBox.Text.Replace(",", "")) + CalCulData;
                    nMoneyBuf[1] = Convert.ToDecimal(TotalTextBox.Text.Replace(",", "")) + CalCulData;
                    break;
                case 2 :    // 출금 증가
                    nMoneyBuf[0] = Convert.ToDecimal(SubTextBox.Text.Replace(",", "")) + CalCulData;
                    nMoneyBuf[1] = Convert.ToDecimal(TotalTextBox.Text.Replace(",", "")) - CalCulData;
                    break;
                case 3:     //입금 감소
                    nMoneyBuf[0] = Convert.ToDecimal(SubTextBox.Text.Replace(",", "")) - CalCulData;
                    nMoneyBuf[1] = Convert.ToDecimal(TotalTextBox.Text.Replace(",", "")) - CalCulData;
                    break;
                case 4:     //출금 감소
                    nMoneyBuf[0] = Convert.ToDecimal(SubTextBox.Text.Replace(",", "")) - CalCulData;
                    nMoneyBuf[1] = Convert.ToDecimal(TotalTextBox.Text.Replace(",", "")) + CalCulData;
                    break;
            }

            SubTextBox.Text = nMoneyBuf[0].ToString("###,###");
            TotalTextBox.Text = nMoneyBuf[1].ToString("###,###");

            //메모리 해제
            nMoneyBuf = null;
        }

        /// <summary>
        /// 그리드 에서 데이터 삭제
        /// </summary>
        /// <param name="MessageData">메시지 전문</param>
        /// 

        private void DataGridDelete(String MessageData)
        {
            MWLogWriter.Instance().LogWrite("전문 수신 Msg : " + MessageData);
            cDataGrid.Rows.Remove(cDataGrid.SelectedRows[0]);
            String[] Data = MessageData.Split('|');
            Decimal MoneyBuf = Convert.ToDecimal(Data[2]);

            if (Data[1].Equals("0"))
            {
                TotalCalculater(3, cInputTotalText, cTotalText, MoneyBuf);
            }
            else if (Data[1].Equals("1"))
            {
                TotalCalculater(4, cOutputTotalText, cTotalText, MoneyBuf);
            }

            MWLogWriter.Instance().LogWrite("cDataGrid View 데이터가 삭제되었습니다. Data Index : " + Data[0]);

            Data = null;
        }

        /// <summary>
        /// 그리드로 데이터 추가
        /// </summary>
        /// <param name="MessageData">전문 내용</param>
        /// 

        private void DataGridInsert(String MessageData)
        {
            MWLogWriter.Instance().LogWrite("전문 수신 Msg : " + MessageData);
            DataRow cDataRow;
            Encoding enc = Encoding.GetEncoding("Shift_JIS");
            String[] Data = MessageData.Split('|');
            String sDateFormat = String.Format("{0:0000}{1:00}{2:00}", Convert.ToInt32(cSDateYearCombo.Text), Convert.ToInt32(cSDateMonthCombo.Text), Convert.ToInt32(cSDateDayCombo.Text));
            int sDate = Convert.ToInt32(sDateFormat);
            sDateFormat = String.Format("{0:0000}{1:00}{2:00}", Convert.ToInt32(cEDateYearCombo.Text), Convert.ToInt32(cEDateMonthCombo.Text), Convert.ToInt32(cEDateDayCombo.Text));
            int eDate = Convert.ToInt32(sDateFormat);
            int Date = Convert.ToInt32(Data[3]);
            if (sDate <= Date && eDate >= Date)
            { 
                Decimal MoneyBuf = Convert.ToDecimal(Data[5]);
                cDataRow = ((DataTable)cDataGrid.DataSource).NewRow();
                cDataRow[0] = Data[0];
                cDataRow[1] = Data[1];
                cDataRow[2] = Date;
                cDataRow[3] = Data[2];
                cDataRow[4] = Data[4];
                cDataRow[5] = MoneyBuf;
                ((DataTable)cDataGrid.DataSource).Rows.Add(cDataRow);

                if (Data[6] == "0")
                {
                    TotalCalculater(1, cInputTotalText, cTotalText, MoneyBuf);
                }
                else if (Data[6] == "1")
                {
                    TotalCalculater(2, cOutputTotalText, cTotalText, MoneyBuf);
                }
            }
            cDataGrid.ClearSelection();
            cDataGrid.Rows[cDataGrid.Rows.Count - 1].Selected = true;
            cDataGrid.FirstDisplayedScrollingRowIndex = cDataGrid.Rows.Count - 1;

            MWLogWriter.Instance().LogWrite("cDataGrid View 데이타가 입력되었습니다. Data Index :" + Data[0]);

            //메모리 해제
            enc = null;
            Data = null;
            cDataRow = null;
        }

        /// <summary>
        /// 시작 날짜 연 콤보 셀렉 체인지 이벤트
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// 

        private void cSDateYearCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cSDateYearCombo.Text != String.Empty && cSDateMonthCombo.Text != String.Empty && cSDateDayCombo.Text != String.Empty)
            {
                DayChange(Convert.ToInt32(cSDateYearCombo.Text), Convert.ToInt32(cSDateMonthCombo.Text), cSDateDayCombo);
            }
        }

        /// <summary>
        /// 시작 날짜 월 콤보 셀렉 체인지 이벤트
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// 

        private void cSDateMonthCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cSDateYearCombo.Text != String.Empty && cSDateMonthCombo.Text != String.Empty && cSDateDayCombo.Text != String.Empty)
            {
                DayChange(Convert.ToInt32(cSDateYearCombo.Text), Convert.ToInt32(cSDateMonthCombo.Text), cSDateDayCombo);
            }
        }

        /// <summary>
        /// 검색 버튼 클릭 이벤트
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// 

        private void cSearchButton_Click(object sender, EventArgs e)
        {
            SearchData();
        }

        /// <summary>
        /// 종료 날짜 연 셀렉 체인지 이벤트
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// 

        private void cEDateYearCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cEDateYearCombo.Text != String.Empty && cEDateMonthCombo.Text != String.Empty && cEDateDayCombo.Text != String.Empty)
            {
                DayChange(Convert.ToInt32(cEDateYearCombo.Text), Convert.ToInt32(cEDateMonthCombo.Text), cEDateDayCombo);
            }
        }

        /// <summary>
        /// 종료 날짜 월 셀렉 체인지 이벤트
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// 

        private void cEDateMonthCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cEDateYearCombo.Text != String.Empty && cEDateMonthCombo.Text != String.Empty && cEDateDayCombo.Text != String.Empty)
            {
                DayChange(Convert.ToInt32(cEDateYearCombo.Text), Convert.ToInt32(cEDateMonthCombo.Text), cEDateDayCombo);
            }
        }

    }//클래스
}//네임스페이스
