﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Windows.Forms;

namespace MW_HouseholdLedger
{
    /// <summary>
    /// 가계부 검색 타입
    /// </summary>
    public enum MWType
    {
        IMPORTMONEY, EXPORTMONEY, TOTALMONEY
    }

    /// <summary>
    /// MSSQL 접속 인스턴스 클래스
    /// </summary>
    /// 


    public class MWSqlDB
    {
        private static MWSqlDB oDBInstance = null;
        private SqlConnection conn = null;
        private SqlCommand cmd = null;

        /// <summary>
        /// 생성자
        /// </summary>
        /// 

        public MWSqlDB()
        {
            conn = new SqlConnection("Data Source=TAMAI-PC\\SQLEXPRESS;Database=MW_Household;User Id=sa;Password=12345");
#if DEBUG
            conn.ConnectionString = "Data Source=TAMAI-PC\\SQLEXPRESS;Database=MW_HouseholdDEV;User Id=sa;Password=12345";
#endif
        }

        /// <summary>
        /// 데이터 검색 메소드
        /// </summary>
        /// <param name="sDate">검색 시작날짜</param>
        /// <param name="eDate">검색 종료날짜</param>
        /// <param name="Where">검색 조건</param>
        /// <param name="DataGrid">그리드 포인터</param>
        /// <returns>[0] - 입금 금액 [1] - 출금금액</returns>
        /// 

        public Decimal[] DataSelect(String sDate, String eDate, StringBuilder Where, DataGridView DataGrid) 
        {
            DataTable DT = new DataTable();
            Decimal[] TotalMoney = new Decimal[2];
            String strqy = String.Format(@"select a.idx,c.MoneyType,convert(decimal(8,0),a.moneyDay),b.Category,a.Contents,a.Money from TBL_AccountBook a
                                            inner join TBL_Category b on a.category = b.idx
                                            inner join TBL_MoneyType c on a.moneytype = c.idx
                                            where a.moneyday between {0} and {1} {2}",
                                            sDate, eDate, Where);
            SqlDataAdapter sda = new SqlDataAdapter();
            sda.SelectCommand = new SqlCommand(strqy, conn);
            sda.Fill(DT);
            DataGrid.DataSource = DT;

            DT = null;
            sda = null;
            strqy = String.Format(@"select MoneyType,sum(Money) from TBL_AccountBook a
                                    where moneyday between {0} and {1} {2}
                                    group by MoneyType",
                                    sDate, eDate, Where);
            cmd = new SqlCommand(strqy, conn);
            try
            {
                conn.Open();
                SqlDataReader dr = cmd.ExecuteReader();
                for (int i = 0; dr.Read(); i++)
                {
                    switch(dr.GetInt32(0))
                    {
                        case 1: TotalMoney[0] = dr.GetDecimal(1); break;
                        case 2: TotalMoney[1] = dr.GetDecimal(1); break;
                    }
                }
                dr = null;
            }
            catch (SqlException)
            {
            }
            finally
            {
                conn.Close();
                cmd = null;
                
            }
            return TotalMoney;
        }

        /// <summary>
        /// 데이터 베이스 값 입력
        /// </summary>
        /// <param name="type">입출금 타입</param>
        /// <param name="category">카테고리 타입 - 재개발</param>
        /// <param name="date">입력 날짜</param>
        /// <param name="Contents">입력 내용</param>
        /// <param name="money">금액 (콤마 없음)</param>
        /// <returns>입력된 IDX 리턴</returns>
        /// 

        public int DataInsert(MWType type, String category, String date, String Contents, String money) 
        {
            int ret = 0;
            String typebuf = "";
            if (type == MWType.IMPORTMONEY)
            {
                typebuf = "1";
            }
            else if (type == MWType.EXPORTMONEY)
            {
                typebuf = "2";
            }

            String strqy = String.Format(@"insert into TBL_AccountBook (moneytype,category,moneyday,contents,money)
                                                                    values('{0}','{1}','{2}',N'{3}','{4}');
                                            select scope_identity();",
                                             typebuf, category, date, Contents, money);
            cmd = new SqlCommand(strqy, conn);
            try
            {
                conn.Open();
                SqlDataReader dr = cmd.ExecuteReader();
                if (dr.Read())
                {
                    ret = Convert.ToInt32(dr.GetValue(0));
                }
                dr = null;
            }
            catch (SqlException ex)
            {
                ret = -1;
            }
            finally
            {
                conn.Close();
                cmd.Dispose();
                cmd = null;
            }
            return ret;
        }

        /// <summary>
        /// 데이터 수정
        /// </summary>
        /// <param name="idx">수정 IDX</param>
        /// <param name="type">입력 타입</param>
        /// <param name="category">카테고리</param>
        /// <param name="date">입력 날짜</param>
        /// <param name="Contents">입력 내용</param>
        /// <param name="money">입력 금액</param>
        /// 

        public void DataModify(String idx,MWType type, String category, String date, String Contents, String money)
        {
            String typebuf = "";
            if (type == MWType.IMPORTMONEY)
            {
                typebuf = "1";
            }
            else if (type == MWType.EXPORTMONEY)
            {
                typebuf = "2";
            }
            String strqy = String.Format(@"update TBL_AccountBook 
                                            set
                                            moneytype = '{0}',
                                            category = '{1}',
                                            moneyday = '{2}',
                                            contents = N'{3}',
                                            money = '{4}'
                                            where idx = '{5}'",
                                            typebuf, category, date, Contents, money,idx);
            cmd = new SqlCommand(strqy, conn);
            try
            {
                conn.Open();
                cmd.ExecuteNonQuery();
            }
            catch (SqlException ex)
            {

            }
            finally
            {
                conn.Close();
                
                cmd.Dispose();
                cmd = null;
            }
        }

        /// <summary>
        /// 데이터 삭제
        /// </summary>
        /// <param name="idx">데이터 IDX</param>
        /// 

        public void DataDelete(String idx)
        {
            String strqy = String.Format(@"delete TBL_AccountBook where idx = '{0}'", idx);
            cmd = new SqlCommand(strqy, conn);
            try
            {
                conn.Open();
                cmd.ExecuteNonQuery();
            }
            catch (SqlException ex)
            {

            }
            finally
            {
                conn.Close();
                
                cmd.Dispose();
                cmd = null;
            }

        }

        /// <summary>
        /// 클래스 인스턴스화
        /// </summary>
        /// <returns></returns>
        /// 

        public static MWSqlDB Instance()
        {
            if (oDBInstance == null)
            {
                oDBInstance = new MWSqlDB();
            }
            return oDBInstance;
        }
    }
}
